// CGBandObj.cpp : Implementation of CCGBandObj

#include "stdafx.h"
#include "CGBand.h"
#include "CGBandObj.h"

// Xtra: additional header files
#include <ExDispID.h>  // For IDispatch::Invoke()


/////////////////////////////////////////////////////////////////////////////
// CCGBandObj


/////////////////////////////////////////////////////////////////////////////
//
// IDispatch Methods
//

//
// Demo how to handle events fired by Internet Explorer.  Here we handle the
// following events:
//
//   DISPID_BEFORENAVIGATE2
//   DISPID_DOCUMENTCOMPLETE
//
STDMETHODIMP CCGBandObj::Invoke(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wFlags,
                                    DISPPARAMS* pDispParams, VARIANT* pVarResult,
                                    EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
	if (IID_NULL != riid)
	{
		return DISP_E_UNKNOWNINTERFACE;
	}

	if (!pDispParams)
	{
		return DISP_E_PARAMNOTOPTIONAL;
	}

	switch (dispIdMember)
	{
		case DISPID_BEFORENAVIGATE2:
			{
				HWND pHWND;

				m_pMainWB->get_HWND((long *)&pHWND);
				::MessageBox(pHWND, "BEFORENAVIGATE2 Event!", "CodeGuru Band", MB_ICONWARNING);
			}
			break;

		case DISPID_DOCUMENTCOMPLETE:
			{
				HWND pHWND;

				m_pMainWB->get_HWND((long *)&pHWND);
				::MessageBox(pHWND, "DOCUMENTCOMPLETE Event!", "CodeGuru Band", MB_ICONWARNING);
			}
			break;

		default:
			return DISP_E_MEMBERNOTFOUND;
	}

	return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
//
// IObjectWithSite Methods
//
STDMETHODIMP CCGBandObj::SetSite(IUnknown* pUnkSite)
{
	//
	// If punkSite is not NULL, a new site is being set.
	//
	if (pUnkSite)
	{
		//
		// If a site is being held, release it and
		// release the in-place object and frame interfaces.
		//
		if (_pSite)
		{
			_pSite->Release();
			_pSite = NULL;
		}

		// Get the parent window.
		IOleWindow* pOleWindow;
		if (SUCCEEDED(pUnkSite->QueryInterface(IID_IOleWindow, (LPVOID*)&pOleWindow)))
		{
			pOleWindow->GetWindow(&_hwndParent);
			pOleWindow->Release();
		}

		_ASSERT(_hwndParent);
		if (!_hwndParent) { return E_FAIL; }

		RECT rc;
		::GetClientRect(_hwndParent, &rc);
		Create(_hwndParent, rc);

		// Get and keep the IInputObjectSite pointer.
		HRESULT hr = pUnkSite->QueryInterface(IID_IInputObjectSite, (LPVOID*)&_pSite);
		_ASSERT(SUCCEEDED(hr));

		//
		// Get the IWebBrowser2 interface of Internet Explorer
		// This is so we can automate the browser.
		//
		IOleCommandTarget* pCmdTarget;
		hr = pUnkSite->QueryInterface(IID_IOleCommandTarget, (LPVOID*)&pCmdTarget);
		if (SUCCEEDED(hr))
		{
			IServiceProvider* pSP;
			hr = pCmdTarget->QueryInterface(IID_IServiceProvider, (LPVOID*)&pSP);

			pCmdTarget->Release();

			if (SUCCEEDED(hr))
			{
				if (m_pMainWB)
				{
					m_pMainWB->Release();
					m_pMainWB = NULL;
				}

				hr = pSP->QueryService(SID_SWebBrowserApp, IID_IWebBrowser2, (LPVOID*)&m_pMainWB);
				_ASSERT(m_pMainWB);

				pSP->Release();

				if (SUCCEEDED(hr))
				{
					//
					// Advise the IWebBrowser2 in the main window that
					// we will be listening to its events.
					//
					// m_pMainWB - pointer to the IWebBrowser2 in the main
					//             window
					//
					AtlAdvise(m_pMainWB, GetUnknown(), DIID_DWebBrowserEvents2, &m_dwCookie);

					// Set the IWebBrowser2 to be automated
					m_pUI->SetBrowserCtrl(m_pMainWB);
				}
			}
		}

	}

	return S_OK;
}

STDMETHODIMP CCGBandObj::GetSite(REFIID riid, void **ppvSite)
{
	*ppvSite = NULL;
	if(_pSite)
	{
		return _pSite->QueryInterface(riid,ppvSite);
	}

	return E_FAIL;
}


/////////////////////////////////////////////////////////////////////////////
//
// IDeskBand Methods
//
STDMETHODIMP CCGBandObj::GetBandInfo(DWORD dwBandID, DWORD dwViewMode,DESKBANDINFO* pdbi)
{
	if(pdbi)
	{
		m_dwBandID = dwBandID;
		m_dwViewMode = dwViewMode;

		if(pdbi->dwMask & DBIM_MINSIZE)
		{
			pdbi->ptMinSize.x = MIN_SIZE_X;
			pdbi->ptMinSize.y = MIN_SIZE_Y;
		}

		if(pdbi->dwMask & DBIM_MAXSIZE)
		{
			pdbi->ptMaxSize.x = -1;
			pdbi->ptMaxSize.y = -1;
		}

		if(pdbi->dwMask & DBIM_INTEGRAL)
		{
			pdbi->ptIntegral.x = 1;
			pdbi->ptIntegral.y = 1;
		}

		if(pdbi->dwMask & DBIM_ACTUAL)
		{
			pdbi->ptActual.x = 0;
			pdbi->ptActual.y = 0;
		}

		//
		// Set the title of the Explorer Bar
		//
		if(pdbi->dwMask & DBIM_TITLE)
		{
			lstrcpyW(pdbi->wszTitle, L"CGBand");
		}

		if(pdbi->dwMask & DBIM_MODEFLAGS)
		{
			pdbi->dwModeFlags = DBIMF_VARIABLEHEIGHT;

		}

		if(pdbi->dwMask & DBIM_BKCOLOR)
		{
			pdbi->dwMask &= ~DBIM_BKCOLOR;

		}
		return S_OK;

	}

	return E_INVALIDARG;
}

